/*
  HC4LED.cpp - Arduino control library for Hitt Consulting HC4LED display
  Copyright (c) 2008 Mikal Hart.  All right reserved.

  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

#include "WConstants.h"
#include <HC4LED.h>

void HC4LED::shift_in_code(unsigned char code)
{
  if (m_blank)
	  digitalWrite(m_blank, LOW); // make sure display is enabled!

  digitalWrite(m_clock, LOW);
  for (int i=0; i<8; ++i)
  {
    digitalWrite(m_data, (code & 0x80) ? HIGH : LOW);
    digitalWrite(m_clock, HIGH);
	// the datasheet says 1ms delay is sufficient, but I have never found this to be the case
	delayMicroseconds(2000);
    digitalWrite(m_clock, LOW);
	delayMicroseconds(2000);
    code <<= 1;
  }
}

unsigned char HC4LED::lookup(char c)
{
  const table_type_1 *pEntry = 0;
  if (c >= 'A' && c <= 'Z')
      pEntry = &alpha_table[c - 'A'];
  else if (c >= 'a' && c <= 'z')
      pEntry = &alpha_table[c - 'a'];
  else if (c >= '0' && c <= '9')
      pEntry = &number_table[c - '0'];

  if (pEntry)
	  return m_upside_down ? pEntry->code2 : pEntry->code1;

  for (int i=0; other_table[i].c; ++i)
    if (other_table[i].c == c)
      return m_upside_down ? other_table[i].code2 : other_table[i].code1;
  
  return 0;
}

/* public interface */

// constructor: saves pin into and orientation
HC4LED::HC4LED(int clock_pin, int data_pin, int blank_pin, bool upside_down) : 
  m_clock(clock_pin),
  m_data(data_pin),
  m_blank(blank_pin),
  m_upside_down(upside_down),
  m_scroll_interval(250)
{
  pinMode(clock_pin, OUTPUT);
  pinMode(data_pin, OUTPUT);
  pinMode(blank_pin, OUTPUT);
}

// use this function to set the time (in ms) between scrolling characters
void HC4LED::set_scroll_interval(unsigned si)
{
  m_scroll_interval = si;
}

// use this function to toggle between orientations
// note: upside_down=true should be the normal usage for scrolling
void HC4LED::set_upside_down(bool upside_down)
{
  m_upside_down = upside_down;
}
	
// clear the display
void HC4LED::clear()
{
  display_text("    ");
}

// all display functions have an optional delay_time parameter which defines
// in milliseconds the amount of time to wait before continuing

// display (static) text - up to 4 characters in the current orientation
void HC4LED::display_text(char text[4], unsigned delay_time /* = 0 */)
{
  if (m_upside_down)
    for (int i=0; i<4; ++i)
      shift_in_code(lookup(text[i]));
  else
    for (int i=3; i>=0; --i)
      shift_in_code(lookup(text[i]));
  if (delay_time) 
      delay(delay_time);
}

// display an unsigned number from 0 to 9999 in the current orientation
void HC4LED::display_number(unsigned short num, unsigned delay_time /* = 0 */)
{
  char sz[5];
  sz[4] = '\0';
  for (int i=3; i>=0; --i)
  {
    sz[i] = (num || i == 3) ? num % 10 + '0' : ' ';
    num /= 10;
  }
  display_text(sz, delay_time);
}

// scroll a string of arbitrary length (really only works with 
// "upside down" orientation)
void HC4LED::scroll_text(char *text, unsigned delay_time /* = 0 */)
{
  clear();
  while (*text)
  {
    shift_in_code(lookup(*text++));
    delay(m_scroll_interval);
  }
  if (delay_time) 
      delay(delay_time);
}

// ----4----
// |       |
// 2       8
// |       |
// |---1---|
// |       |
// 40h   10h
// |       |
// ---20h---

/* static */
const HC4LED::table_type_1 HC4LED::alpha_table[] = 
{
  {  /* 'A', */ 0x5F, 0x7B, },
  {  /* 'B', */ 0x73, 0x1F, },
  {  /* 'C', */ 0x66, 0x3C, },
  {  /* 'D', */ 0x79, 0x4F, },
  {  /* 'E', */ 0x67, 0x3D, },
  {  /* 'F', */ 0x47, 0x39, },
  {  /* 'G', */ 0x3F, 0x77, },
  {  /* 'H', */ 0x5B, 0x5B, },
  {  /* 'I', */ 0x18, 0x42, },
  {  /* 'J', */ 0x78, 0x4E, },
  {  /* 'K', */ 0x63, 0x1D, },
  {  /* 'L', */ 0x62, 0x1C, },
  {  /* 'M', */ 0x5E, 0x7A, },
  {  /* 'N', */ 0x51, 0x0B, },
  {  /* 'O', */ 0x7E, 0x7E, },
  {  /* 'P', */ 0x4F, 0x79, },
  {  /* 'Q', */ 0x1F, 0x73, },
  {  /* 'R', */ 0x41, 0x09, },
  {  /* 'S', */ 0x37, 0x37, },
  {  /* 'T', */ 0x1C, 0x62, },
  {  /* 'U', */ 0x7A, 0x5E, },
  {  /* 'V', */ 0x7A, 0x5E, },
  {  /* 'W', */ 0x7A, 0x5E, },
  {  /* 'X', */ 0x5B, 0x5B, },
  {  /* 'Y', */ 0x3B, 0x57, },
  {  /* 'Z', */ 0x6D, 0x6D, },
};

const HC4LED::table_type_1 HC4LED::number_table[] = 
{
  {  /* '0', */ 0x7E, 0x7E, },
  {  /* '1', */ 0x18, 0x42, },
  {  /* '2', */ 0x6D, 0x6D, },
  {  /* '3', */ 0x3D, 0x67, },
  {  /* '4', */ 0x1B, 0x53, },
  {  /* '5', */ 0x37, 0x37, },
  {  /* '6', */ 0x77, 0x3F, },
  {  /* '7', */ 0x1C, 0x62, },
  {  /* '8', */ 0x7F, 0x7F, },
  {  /* '9', */ 0x3F, 0x77, },
};

const HC4LED::table_type_2 HC4LED::other_table[] = 
{
  {  ' ', 0x00, 0x00, },
  {  '-', 0x01, 0x01, },
  {  '\'', 0x08, 0x40, },
  {  0, 0, 0 },
};
    
