/*
  HC4LED.h - Arduino control library for Hitt Consulting HC4LED display
  Copyright (c) 2008 Mikal Hart.  All right reserved.

  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

  The HC4LED is an inexpensive and easy to use, 4-digit, 7-segment, 
  display module, useful for microcontroller projects that need to 
  display small numbers or short bits of text. It's also useful as 
  a debugging device in situations where a serial monitor is not 
  available or practical. 

  The module has some drawbacks compared to the popular LCD-style displays. 
  It only has four available characters/digits, and these are made from 
  7-segment LED-type displays which seriously limit its ability to render 
  certain letters and cases.  Because you program the device using a serial 
  "shift in" algorithm with a considerably slow clock (2ms), display updates 
  can look flickery and slow. Finally, there is the concern that these 
  devices seem to be remaindered. The supplier linked above seems, for example, 
  to sell only used ones that have been "pulled from working systems". 

  That said, the devices are quite cheap (~$4 USD) and easy to program using this
  library.  Most importantly, each only requires 2 (or 3) of your precious Arduino 
  pins. 

  Sample usage:

#include <HC4LED.h>

#define DISP_DATA_PIN 4
#define DISP_CLOCK_PIN 5
#define DISP_BLANK_PIN 7

HC4LED display(DISP_CLOCK_PIN, DISP_DATA_PIN, DISP_BLANK_PIN, true);

void setup()
{
  display.scroll_text("Hello");
}

void loop()
{
  static unsigned short last = 0;
  unsigned short next = (unsigned short)(millis() / 1000);
  if (next != last)
    display.display_number(last = next);
}

*/

class HC4LED
{
  // Notes on cable connection to HC4LED
  // Wire 1: Data (blue)
  // Wire 2: Clock (blue)
  // Wire 3: not connected
  // Wire 4: Blank (blue)
  // Wire 5: Gnd (blue)
  // Wire 6: +5V (white)
  
  private:
    int m_clock, m_data, m_blank; // pin information
    bool m_upside_down;           // true if displaying upside down
    unsigned m_scroll_interval;   // ms between character advancing
    struct table_type_1	          // lookup table #1
    {
      unsigned char code1;  // right side up code
      unsigned char code2;  // upside down code
    };
    struct table_type_2           // lookup table #2
    {
      char c;               // character to match
      unsigned char code1;  // right side up code
      unsigned char code2;  // upside down code
    };
    static const table_type_1 alpha_table[];
    static const table_type_1 number_table[];
    static const table_type_2 other_table[];

    void shift_in_code(unsigned char code);
    unsigned char lookup(char c);

  public:
    // constructor: saves pin into and orientation
    HC4LED(int clock_pin, int data_pin, int blank_pin, bool upside_down);

    // use this function to set the time (in ms) between scrolling characters
    void set_scroll_interval(unsigned si);

    // use this function to toggle between orientations
    // note: upside_down=true should be the normal usage for scrolling
    void set_upside_down(bool upside_down);

    // clear the display
    void clear();

    // all display functions have an optional delay_time parameter which defines
    // in milliseconds the amount of time to wait before continuing

    // display (static) text - up to 4 characters in the current orientation
    void display_text(char text[4], unsigned delay_time = 0);

    // display an unsigned number from 0 to 9999 in the current orientation
    void display_number(unsigned short num, unsigned delay_time = 0);

    // scroll a string of arbitrary length (really only works with 
    // "upside down" orientation)
    void scroll_text(char *text, unsigned delay_time = 0);
};

